<?php

namespace FiloBlu\CheckoutExtend\Api\Plugin\Sales;

use Magento\Quote\Model\Quote\Address as QuoteAddress;
use Magento\Framework\App\ObjectManager;

class OrderManagementInterface {

    protected $_subscriberFactory;
    protected $_helper;
    protected $_logger;
    protected $_quoteFactory;
    protected $_request;
    protected $_checkoutSession;
    protected $objectCopyService;
    protected $addressFactory;
    protected $regionFactory;
    protected $customerFactory;
    protected $session;
    protected $accountManagement;
    protected $cookieMetadataManager;
    protected $cookieMetadataFactory;
    protected $orderRepository;
    protected $_eventManager;
    protected $coupon;
    protected $couponUsage;
    protected $logger;
    protected $ruleCustomerFactory;

    /**
     * OrderManagementInterface constructor.
     * @param \Magento\Newsletter\Model\SubscriberFactory $subscriberFactory
     * @param \FiloBlu\CheckoutExtend\Helper\Data $helper
     * @param \Magento\Quote\Model\QuoteFactory $quoteFactory
     * @param \Magento\Framework\App\RequestInterface $request
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Framework\DataObject\Copy $objectCopyService
     * @param \Magento\Customer\Api\Data\AddressInterfaceFactory $addressFactory
     * @param \Magento\Customer\Api\Data\RegionInterfaceFactory $regionFactory
     * @param \Magento\Customer\Api\Data\CustomerInterfaceFactory $customerFactory
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Customer\Api\AccountManagementInterface $accountManagement
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\Framework\Event\ManagerInterface $eventManager
     * @param \Magento\SalesRule\Model\Coupon $coupon
     * @param \Magento\SalesRule\Model\ResourceModel\Coupon\Usage $couponUsage
     * @param \Magento\SalesRule\Model\Rule\CustomerFactory $ruleCustomerFactory
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \Magento\Newsletter\Model\SubscriberFactory $subscriberFactory, \FiloBlu\CheckoutExtend\Helper\Data $helper, \Magento\Quote\Model\QuoteFactory $quoteFactory, \Magento\Framework\App\RequestInterface $request, \Magento\Checkout\Model\Session $checkoutSession, \Magento\Framework\DataObject\Copy $objectCopyService, \Magento\Customer\Api\Data\AddressInterfaceFactory $addressFactory, \Magento\Customer\Api\Data\RegionInterfaceFactory $regionFactory, \Magento\Customer\Api\Data\CustomerInterfaceFactory $customerFactory, \Magento\Customer\Model\Session $customerSession, \Magento\Customer\Api\AccountManagementInterface $accountManagement, \Magento\Sales\Api\OrderRepositoryInterface $orderRepository, \Magento\Framework\Event\ManagerInterface $eventManager, \Magento\SalesRule\Model\Coupon $coupon, \Magento\SalesRule\Model\ResourceModel\Coupon\Usage $couponUsage, \Magento\SalesRule\Model\Rule\CustomerFactory $ruleCustomerFactory, \Psr\Log\LoggerInterface $logger
    ) {
        $this->_subscriberFactory = $subscriberFactory;
        $this->_helper = $helper;
        $this->_quoteFactory = $quoteFactory;
        $this->_request = $request;
        $this->_checkoutSession = $checkoutSession;
        $this->objectCopyService = $objectCopyService;
        $this->addressFactory = $addressFactory;
        $this->regionFactory = $regionFactory;
        $this->customerFactory = $customerFactory;
        $this->session = $customerSession;
        $this->accountManagement = $accountManagement;
        $this->orderRepository = $orderRepository;
        $this->_eventManager = $eventManager;
        $this->coupon = $coupon;
        $this->couponUsage = $couponUsage;
        $this->ruleCustomerFactory = $ruleCustomerFactory;
        $this->logger = $logger;
    }

    /**
     * @param $interceptor
     * @param $order
     * @return mixed
     */
    public function afterPlace($interceptor, $order) {

        $quote = null;

        if ($order->getQuoteId()) {
            $quote = $this->_quoteFactory->create()->load($order->getQuoteId());
        }

        // Create Customer Account
        try {
            if ($this->_helper->getConfigValue("filoblu_checkoutextend_section/registration/enabled") && !$order->getCustomerId() && !$this->session->isLoggedIn() && $quote && $quote->getCheckoutRegistration()) {

//                $this->session->regenerateId();

                $customerData = $this->objectCopyService->copyFieldsetToTarget(
                    'order_address', 'to_customer', $order->getBillingAddress(), []
                );

                $addresses = $order->getAddresses();
                foreach ($addresses as $address) {
                    $addressData = $this->objectCopyService->copyFieldsetToTarget(
                        'order_address', 'to_customer_address', $address, []
                    );
                    /** @var \Magento\Customer\Api\Data\AddressInterface $customerAddress */
                    $customerAddress = $this->addressFactory->create(['data' => $addressData]);
                    switch ($address->getAddressType()) {
                        case QuoteAddress::ADDRESS_TYPE_BILLING:
                            $customerAddress->setIsDefaultBilling(true);
                            break;
                        case QuoteAddress::ADDRESS_TYPE_SHIPPING:
                            $customerAddress->setIsDefaultShipping(true);
                            break;
                    }

                    if (is_string($address->getRegion())) {
                        /** @var \Magento\Customer\Api\Data\RegionInterface $region */
                        $region = $this->regionFactory->create();
                        $region->setRegion($address->getRegion());
                        $region->setRegionCode($address->getRegionCode());
                        $region->setRegionId($address->getRegionId());
                        $customerAddress->setRegion($region);
                    }
                    $customerData['addresses'][] = $customerAddress;
                }

                $customerPwd = $this->_checkoutSession->getRegistrationPwd();

                $customer = $this->customerFactory->create(['data' => $customerData]);
                $account = $this->accountManagement->createAccount($customer, $customerPwd);

                $this->_eventManager->dispatch(
                    'customer_register_success', ['account_controller' => null, 'customer' => $account]
                );

                if ($account->getId()) {
                    $order->setCustomerId((int) $account->getId());
                    $order->setCustomerIsGuest(false);
                    $order->setCustomerFirstname($account->getFirstname());
                    $order->setCustomerLastname($account->getLastname());
                    $order->setCustomerGroupId((int)$account->getGroupId());
                    $this->orderRepository->save($order);
                    $this->session->setCustomerMustLogin((int) $account->getId());
                }

                $this->_checkoutSession->setRegistrationPwd(null);

                //Writes Coupon Usages
                if (!empty($order->getCouponCode())) {
                    $this->updateCouponUsages($order, (int)$account->getId());
                }

                //Writes Rule usages
                if(!empty($order->getAppliedRuleIds())) {
                    $this->addCustomerRule($order,(int)$account->getId());
                }

            }
        } catch (\Exception $ex) {
            $this->logger->error($ex->getMessage());
        }

        // Subscribe to Newsletter
        try {


            if ($this->_helper->getConfigValue("filoblu_checkoutextend_section/newsletter/enabled") && $quote && $quote->getIsSubscribeForNewsletter()) {

                $billingAddress = $order->getBillingAddress();

                if ($order->getCustomerIsGuest()) {

                    // Set newsletter subscriber data in request
                    $this->_request->setPostValue('firstname', $billingAddress->getFirstname());
                    $this->_request->setPostValue('lastname', $billingAddress->getLastname());
                    $this->_request->setPostValue('country_id', $billingAddress->getCountryId());
                    $this->_request->setPostValue('phone', $billingAddress->getTelephone());
                    $this->_request->setPostValue('source', "CHECKOUT");
                    $this->_request->setPostValue('language', $this->_helper->getCurrentLocale());
                    $this->_request->setPostValue('website_name', $this->_helper->getCurrentWebsiteName());

                    // Fields not set: dob - gender - collections - birthday

                    $this->_subscriberFactory->create()->subscribe($order->getCustomerEmail());
                } else {

                    // Set newsletter subscriber data in request
                    $this->_request->setPostValue('firstname', ($order->getCustomerFirstname() ?: $billingAddress->getFirstname()));
                    $this->_request->setPostValue('lastname', ($order->getCustomerLastname() ?: $billingAddress->getLastname()));
                    $this->_request->setPostValue('country_id', $billingAddress->getCountryId());
                    $this->_request->setPostValue('phone', $billingAddress->getTelephone());
                    $this->_request->setPostValue('source', "CHECKOUT");
                    $this->_request->setPostValue('language', $this->_helper->getCurrentLocale());
                    $this->_request->setPostValue('website_name', $this->_helper->getCurrentWebsiteName());

                    // Fields not set: dob - gender - collections - birthday

                    $this->_subscriberFactory->create()->subscribeCustomerById((int) $order->getCustomerId());
                }
            }
        } catch (\Exception $ex) {
            $this->logger->error($ex->getMessage());
        }

        // Request An Invoice
        try {

            if ($this->_helper->getConfigValue("filoblu_checkoutextend_section/invoice/enabled") && $quote && $quote->getCheckoutInvoice()) {
                $order->setCheckoutInvoice($quote->getCheckoutInvoice());
                $order->setCheckoutInvoiceFiscalCode($quote->getCheckoutInvoiceFiscalCode());
                $order->setCheckoutInvoiceVatNumber($quote->getCheckoutInvoiceVatNumber());
                $order->setCheckoutInvoiceBusinessName($quote->getCheckoutInvoiceBusinessName());
                $order->setCheckoutInvoicePec($quote->getCheckoutInvoicePec());
                $order->setCheckoutInvoiceSdi($quote->getCheckoutInvoiceSdi());
                $this->orderRepository->save($order);
            }
        } catch (\Exception $ex) {
            $this->logger->error($ex->getMessage());
        }


        return $order;
    }

    /**
     * @return mixed
     */
    private function getCookieManager() {
        if (!$this->cookieMetadataManager) {
            $this->cookieMetadataManager = ObjectManager::getInstance()->get(
                \Magento\Framework\Stdlib\Cookie\PhpCookieManager::class
            );
        }
        return $this->cookieMetadataManager;
    }

    /**
     * @return mixed
     */
    private function getCookieMetadataFactory() {
        if (!$this->cookieMetadataFactory) {
            $this->cookieMetadataFactory = ObjectManager::getInstance()->get(
                \Magento\Framework\Stdlib\Cookie\CookieMetadataFactory::class
            );
        }
        return $this->cookieMetadataFactory;
    }

    /**
     * @param Order $subject
     * @param int $customerId
     */
    private function updateCouponUsages($subject, $customerId)
    {
        $this->coupon->load($subject->getCouponCode(), 'code');
        if ($this->coupon->getId()) {
            $this->couponUsage->updateCustomerCouponTimesUsed($customerId, $this->coupon->getId());
        }
    }

    /**
     * @param $order
     * @param $customerId
     */
    private function addCustomerRule($order, $customerId) {
        $rules = explode(',',$order->getAppliedRuleIds());
        if(count($rules)){
            foreach($rules as $rule) {
                $this->updateCustomerRuleUsages($customerId,$rule);
            }
        }
    }

    /**
     * Update the number of rule usages per customer.
     *
     * @param int $ruleId
     * @param int $customerId
     */
    private function updateCustomerRuleUsages($customerId, $ruleId)
    {
        /** @var \Magento\SalesRule\Model\Rule\Customer $ruleCustomer */
        $ruleCustomer = $this->ruleCustomerFactory->create();
        $ruleCustomer->loadByCustomerRule($customerId, $ruleId);
        $ruleCustomer->setCustomerId($customerId)->setRuleId($ruleId)->setTimesUsed(1);
        $ruleCustomer->save();
    }

}